#include <stdio.h>
#include "SDL.h"

SDL_Surface *tempScreenSeg;

const Uint8 pcxpal[] = /* [1..PCXnum] */
{ 0, 7, 5, 8, 10, 5, 18, 19, 19, 20, 21, 22, 5};

SDL_Color palettes[100][256];

SDL_Color palette[256];

void JE_loadPals( void )
{
	FILE *f;
	int i;

	int palNum = 0;

	f = fopen("palette.dat", "rb");
	while (palNum < 100)
	{
		for (i = 0; i < 256; i++)
		{
			palettes[palNum][i].r = getc(f) << 2;
			palettes[palNum][i].g = getc(f) << 2;
			palettes[palNum][i].b = getc(f) << 2;
		}
		palNum++;
	}
	fclose(f);
}

#define MAX_TABLE         8
#define MAXIMUM_SHAPE     151
Uint8 maxShape[MAX_TABLE];
Uint16 shapeX[MAX_TABLE][MAXIMUM_SHAPE], shapeY[MAX_TABLE][MAXIMUM_SHAPE], shapeSize[MAX_TABLE][MAXIMUM_SHAPE];
Uint8 *shapeArray[MAX_TABLE][MAXIMUM_SHAPE];
Uint8 shapeExist[MAX_TABLE][MAXIMUM_SHAPE];

void JE_newLoadShapesB( Uint8 table, FILE *f )
{
	Uint16 temp;
	
	fread(&temp, sizeof(Uint16), 1, f);
	
	maxShape[table] = temp;
	
	for (int i = 0; i < maxShape[table]; i++)
	{
		shapeExist[table][i] = getc(f);
		
		if (shapeExist[table][i])
		{
			fread(&shapeX   [table][i], sizeof(Uint16), 1, f);
			fread(&shapeY   [table][i], sizeof(Uint16), 1, f);
			fread(&shapeSize[table][i], sizeof(Uint16), 1, f);
			
			shapeArray[table][i] = malloc(shapeX[table][i] * shapeY[table][i]);
			
			fread(shapeArray[table][i], sizeof(Uint8), shapeSize[table][i], f);
		}
	}
}

void JE_loadCompShapesB( Uint8 **shapes, FILE *f, Uint16 shapeSize )
{
	*shapes = malloc(shapeSize);
	fread(*shapes, sizeof(Uint8), shapeSize, f);
}

void JE_newDrawCShapeNum( Uint8 table, Uint8 shape, Uint16 x, Uint16 y )
{
	Uint16 xsize, ysize;
	Uint16 xloop = 0, yloop = 0;
	Uint8 *p; /* shape pointer */
	Uint32 *s; /* screen pointer, 8-bit specific */

	if ((shape-- /* re-index */ > maxShape[table]) || (!shapeExist[table][shape]) || (shape == 255))
	{
		//exit(99); /* pascalism */
	}

	xsize = shapeX[table][shape]; ysize = shapeY[table][shape];

	s = (Uint32 *)tempScreenSeg->pixels;
	s = (Uint32 *)( (Uint8 *)s + y * tempScreenSeg->pitch + x );

	for (p = shapeArray[table][shape]; yloop < ysize; p++)
	{
		switch (*p)
		{
			case 255: /* p transparent pixels */
				p++;
				s += *p; xloop += *p;
				break;
			case 254: /* next y */
				s -= xloop; xloop = 0;
				s = (Uint32 *)( (Uint8 *)s + tempScreenSeg->pitch ); yloop++;
				break;
			case 253: /* 1 transparent pixel */
				s++; xloop++;
				break;
			default:  /* set a pixel */
				if ((void *)s >= tempScreenSeg->pixels)
					*s = palette[*p].r << 16 | palette[*p].g << 8 | palette[*p].b;
				s++; xloop++;
				break;
		}
		if (xloop == xsize)
		{
			s -= xloop; xloop = 0;
			s = (Uint32 *)( (Uint8 *)s + tempScreenSeg->pitch); yloop++;
		}
	}
}

void JE_drawShape2( int x, int y, int s_, Uint8 *shape )
{
	Uint8 *p; /* shape pointer */
	Uint32 *s; /* screen pointer, 8-bit specific */

	int i;

	s = (Uint32 *)tempScreenSeg->pixels;
	s = (Uint32 *)( (Uint8 *)s + y * tempScreenSeg->pitch + x );

	p = shape;
	p += SDL_SwapLE16(((Uint16 *)p)[s_ - 1]);

	while (*p != 0x0f)
	{
		s += *p & 0x0f;
		i = (*p & 0xf0) >> 4;
		if (i)
		{
			while (i--)
			{
				p++;
				if ((void *)s >= tempScreenSeg->pixels)
					*s = palette[*p].r << 16 | palette[*p].g << 8 | palette[*p].b;
				s++;
			}
		} else {
			s -= 12;
			s = (Uint32 *)( (Uint8 *)s + tempScreenSeg->pitch);
		}
		p++;
	}
}

Uint8 *shapesC1, *shapes9, *eShapes6, *eShapes5, *shapesW2;

void JE_loadMainShapeTables( char *shpfile )
{
	FILE *f = fopen(shpfile, "rb");
	
	Uint16 shpNumb;
	Uint32 shpPos[12 + 1]; // +1 for storing file length
	
	fread(&shpNumb, sizeof(shpNumb), 1, f);
	
	for (int i = 0; i < shpNumb; i++)
	{
		fread(&shpPos[i], sizeof(*shpPos), 1, f);
	}
	fseek(f, 0, SEEK_END);
	shpPos[shpNumb] = ftell(f);
	
	int i;
	// fonts, interface, option sprites
	for (i = 0; i < 7; i++)
	{
		fseek(f, shpPos[i], SEEK_SET);
		JE_newLoadShapesB(i, f);
	}
	
	Uint16 shapesC1Size, shapes9Size, eShapes6Size, eShapes5Size, shapesW2Size;
	
	// player shot sprites
	shapesC1Size = shpPos[i + 1] - shpPos[i];
	JE_loadCompShapesB(&shapesC1, f, shapesC1Size);
	i++;
	
	// player ship sprites
	shapes9Size = shpPos[i + 1] - shpPos[i];
	JE_loadCompShapesB(&shapes9 , f, shapes9Size);
	i++;
	
	// power-up sprites
	eShapes6Size = shpPos[i + 1] - shpPos[i];
	JE_loadCompShapesB(&eShapes6, f, eShapes6Size);
	i++;
	
	// coins, datacubes, etc sprites
	eShapes5Size = shpPos[i + 1] - shpPos[i];
	JE_loadCompShapesB(&eShapes5, f, eShapes5Size);
	i++;
	
	// more player shot sprites
	shapesW2Size = shpPos[i + 1] - shpPos[i];
	JE_loadCompShapesB(&shapesW2, f, shapesW2Size);
	
	fclose(f);
}

int main( int argc, char *argv[] )
{
	JE_loadPals();
	
	JE_loadMainShapeTables("tyrianc.shp");
	
	SDL_Surface *VGAScreen = SDL_SetVideoMode(228, 224, 32, SDL_SWSURFACE | SDL_HWPALETTE);
	
	for (int i = 0; i < MAX_TABLE; i++)
	{
		for (int j = 0; j < maxShape[i]; j++)
		{
			int pal[MAX_TABLE] = { 0, 0, 0, 17, 0, 0, 0, 0 };
			memcpy(palette, palettes[pal[i]], 256 * sizeof(SDL_Color));
			
			if (i == 3 && j == 146)
				memcpy(palette, palettes[8], 256 * sizeof(SDL_Color));
			
			if (i == 4)
			{
				const Uint8 facepal[12] = /* [1..12] */
				{ 1, 2, 3, 4, 6, 9, 11, 12, 16, 13, 14, 15};
				memcpy(palette, palettes[facepal[j]], 256 * sizeof(SDL_Color));
			}
			
			tempScreenSeg = SDL_CreateRGBSurface(SDL_SWSURFACE, shapeX[i][j], shapeY[i][j], 32, 0, 0, 0, 0);
			
			SDL_FillRect(tempScreenSeg, NULL, 0xff00ff);
			
			JE_newDrawCShapeNum(i, j + 1, 0, 0);
			
			char foo[100];
			sprintf(foo, "tyrianc.shp-%02d-%03d.bmp", i, j);
			SDL_SaveBMP(tempScreenSeg, foo);
			
			//SDL_Flip(VGAScreen);
			//SDL_Delay(50);
		}
	}
	
	tempScreenSeg = VGAScreen;
	
	for (int sh = 0; sh < 5; sh++)
	{
		Uint8 *shapefoos[] = { shapesC1, shapes9, eShapes6, eShapes5, shapesW2 };
		Uint8 *shapefoo = shapefoos[sh];
		
		SDL_FillRect(tempScreenSeg, NULL, 0xff00ff);
		
		for (int y = 0; y < 16; y++)
		{
			for (int x = 0; x < 19; x++)
			{
				int i = y * 19 + x + 1;
				
				JE_drawShape2(x * 12 * 4, y * 14, i, shapefoo);
			}
		}
		
		char foo[100];
		sprintf(foo, "tyrianc.shp-%02d.bmp", sh + MAX_TABLE);
		SDL_SaveBMP(tempScreenSeg, foo);
	}
	
	return 0;
}
